using AutoSproc;
using System.Data;
using System.Data.SqlClient;
using System;
using System.Xml;
namespace mobileDAL
{
	#region Interfs
	/// <summary>
	/// Interfaz utilizada para utilizar AutoSproc como medio de almacenamiento y consulta a base de datos
	/// </summary>
	public interface IEntidad:ISprocBase
	{
		DataSet Entidad_Select(int entidad);

		DataSet Entidad_Select();

		int Entidad_Delete(int entidad);

		int Entidad_Save(
			ref int entidad,
			int pais,
			string descripcion,
			ref int ultimaAct);
	}
	#endregion //Interfs

	#region Clase
	/// <summary>
	/// Clase utilizada para controlar el acceso a datos asi como la persistencia de Entidad
	/// </summary>
	public class Entidad
	{
		#region Atributos
		private IEntidad proc;	
		private SqlConnection conn=null;
		#endregion //Atributos

		#region Constructor, Destructor
		/// <summary>
		/// Destructor
		/// </summary>
		~Entidad()
		{
			try
			{
				if(conn!=null)
					conn.Dispose();
			}
			catch{}
		}

		/// <summary>
		/// Constructor protegido debido a la utilizacion del patron singleton de gamma para obligar a utilizar una sola instancia de la clase
		/// </summary>
		public Entidad()
		{
			try
			{
				conn=this.GetConnection();		
				
				object imp = null;
				imp = SprocFactory.CreateInstance(typeof (IEntidad), DBProvider.SQLServer);
				
				proc = (IEntidad) imp;
				proc.Connection = conn;
				
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
		}
		#endregion //Constructor, Destructor

		#region Mtodos Privados
		private SqlConnection GetConnection()
		{	
			
//			if(conn==null)
//			{					
				conn=new SqlConnection (Entity.Parametros.GetInstance().Conecction);;										
//			}		

			return conn;
		}
		/// <summary>
		/// Elimina los elementos representados en la lista de la base de datos
		/// </summary>
		/// <param name="listaEntidades">Entity.ListaEntidades representa los elementos a eliminar</param>
		private void Borrar(Entity.ListaEntidades listaEntidades)
		{	
			try
			{
				
				Entity.ListaEntidades listaBorrados= new Entity.ListaEntidades();			
				listaBorrados=(Entity.ListaEntidades)listaEntidades.GetDeleted();
				foreach(Entity.Entidad elemento in listaBorrados)				
				{				
					proc.Entidad_Delete(elemento.Estado);
				}

			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			
		}
		
		#endregion //Mtodos Privados

		#region Mtodos Pblicos
		/// <summary>
		/// Obtiene un System.string que indica la respuesta de la aplicacin del mtodo al almacenar la informacin en la base de datos
		/// </summary>
		/// <param name="listaEntidades">Entity.ListaEntidades representa los elementos a guardar</param>
		/// <returns>System.string</returns>
		public string Guardar(ref Entity.ListaEntidades listaEntidades)
		{
			try
			{
				if(listaEntidades.HasDeleted())
				{
					this.Borrar(listaEntidades);
				}
							
				foreach(Entity.Entidad elemento in listaEntidades)
				{
					int codigo;
					int ultimaActualizacion;
					if(elemento.HasChanges())
					{
						codigo=elemento.Estado;
						ultimaActualizacion=elemento.UltimaActualizacion;
						proc.Entidad_Save(ref codigo, elemento.Pais.PaisId, elemento.Descripcion, ref ultimaActualizacion);
						elemento.Estado=codigo;
						elemento.UltimaActualizacion=ultimaActualizacion;
					}
				}
				listaEntidades.AcceptChanges();
				
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			return "ok";
		}

		/// <summary>
		/// Obtiene una Entity.ListaEntidades
		/// </summary>
		/// <returns>una entidad Entity.ListaEntidades</returns>
		public Entity.ListaEntidades TraerEntidades()
		{	
			Entity.ListaEntidades listaEntidades=new Entity.ListaEntidades();
			try
			{				
				Entity.ListaPaises listaPaises=new mobileDAL.Pais().TraerPaises();
				DataSet dsEntidades = proc.Entidad_Select();
				Entity.Entidad elemento;				
				foreach(DataRow ren in dsEntidades.Tables[0].Rows)
				{
					elemento=new Entity.Entidad();
					elemento.Estado=int.Parse(ren["entidad"].ToString());
					elemento.Pais=listaPaises.FindByPrimaryKey(int.Parse(ren["pais"].ToString()));
					elemento.Descripcion=ren["descripcion"].ToString();
					elemento.UltimaActualizacion=int.Parse(ren["ultimaAct"].ToString());
					
					listaEntidades.Add(elemento);
				}
				
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			listaEntidades.AcceptChanges();
			return listaEntidades;
		}

		/// <summary>
		/// Obtiene una Entity.Entidad segn el parmetro recibido
		/// </summary>
		/// <param name="entidadId">System.String representa la Entidad</param>
		/// <returns>una entidad Entity.Entidad</returns>
		public Entity.Entidad TraerEntidades(int entidadId)
		{	
			Entity.Entidad elemento=null;
			DataSet dsElementos=null;
			Entity.ListaPaises listaPaises=new mobileDAL.Pais().TraerPaises();

			try
			{				
				dsElementos=proc.Entidad_Select(entidadId);
				foreach(System.Data.DataRow row in dsElementos.Tables[0].Rows)
				{
					elemento = new Entity.Entidad();
					elemento.Estado=int.Parse(row["entidad"].ToString());
					elemento.Pais=listaPaises.FindByPrimaryKey(int.Parse(row["pais"].ToString()));
					elemento.Descripcion=row["descripcion"].ToString();
					elemento.UltimaActualizacion=int.Parse(row["UltimaAct"].ToString());						
						
						
					elemento.AcceptChanges();
				}
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			return elemento;
		}
		
		#endregion //Mtodos Pblicos
	}

	#endregion //Clase
}
