using AutoSproc;
using System.Data;
using System.Data.SqlClient;
using System;
using System.Xml;
namespace mobileDAL
{
	/// <summary>
	/// Interfaz utilizada para utilizar AutoSproc como medio de almacenamiento y consulta a base de datos
	/// </summary>
	public interface IPais:ISprocBase
	{
		DataSet Paises_Select();
		DataSet Paises_Select(int pais);
		int	Paises_Delete(int pais);
		int Paises_Save(ref int pais, string  descripcion, ref int ultimaAct);
	}

	/// <summary>
	/// Clase utilizada para controlar el acceso a datos asi como la persistencia de Pais
	/// </summary>
	public class Pais
	{
		private IPais proc;		
		private SqlConnection conn=null;
		//private static volatile Pais instance=null;
		~Pais()
		{
			try
			{
				if(conn!=null)
					conn.Dispose();
			}
			catch{}
		}
		/// <summary>
		/// Constructor protegido debido a la utilizacion del patron singleton de gamma para obligar a utilizar una sola instancia de la clase
		/// </summary>
		public Pais()
		{
			try
			{
				SqlConnection conn = this.GetConnection();
				//conn.Open ();
				
				object imp = null;
				imp = SprocFactory.CreateInstance(typeof (IPais), DBProvider.SQLServer);
				
				proc = (IPais) imp;
				proc.Connection = conn;
				
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
		}

		/// <summary>
		/// Obtiene un SqlConnection de acuerdo a los parametros de configuracion del archivo Xml
		/// </summary>
		/// <returns><see cref="SqlConnection"/></returns>
		private SqlConnection GetConnection()
		{
            conn=new SqlConnection (Entity.Parametros.GetInstance().Conecction);
			return conn;
		}

		/// <summary>
		/// Elimina los elementos representados en la lista de la base de datos
		/// </summary>
		/// <param name="listaPaises">Entity.ListaPaises representa los elementos a eliminar</param>
		private void Borrar(Entity.ListaPaises listaPaises)
		{	
			try
			{
				Entity.ListaPaises listaBorrados= new Entity.ListaPaises();
				listaBorrados=(Entity.ListaPaises)listaPaises.GetDeleted();
				foreach(Entity.Pais elemento in listaBorrados)				
				{				
					proc.Paises_Delete(elemento.PaisId);
				}
				
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
		}
		
		/// <summary>
		/// Obtiene un System.string que indica la respuesta de la aplicacin del mtodo al almacenar la informacin en el DBF
		/// </summary>
		/// <param name="listaPaises">Entity.ListaPaises representa los elementos a guardar</param>
		/// <returns>System.string</returns>
		public string Guardar(Entity.ListaPaises listaPaises)
		{
			try
			{
				if(listaPaises.HasDeleted())
				{
					this.Borrar(listaPaises);
				}
							
				foreach(Entity.Pais elemento in listaPaises)
				{
					int codigo;
					int ultimaActualizacion;
					if(elemento.HasChanges())
					{
						codigo=elemento.PaisId;
						ultimaActualizacion=elemento.UltimaActualizacion;
						proc.Paises_Save(ref codigo, elemento.Descripcion, ref ultimaActualizacion);
						elemento.PaisId=codigo;
						elemento.UltimaActualizacion=ultimaActualizacion;
						
					}
				}
				listaPaises.AcceptChanges();
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			return "ok";
		}

		/// <summary>
		/// Obtiene una Entity.ListaPaises con la informacin de la tabla
		/// </summary>
		/// <returns>una entidad Entity.ListaPaises</returns>
		public Entity.ListaPaises TraerPaises()
		{	
			Entity.ListaPaises listaPaises=new Entity.ListaPaises();
			
			try
			{
				DataSet dsElementos = proc.Paises_Select();
				Entity.Pais elemento;
				foreach(DataRow ren in dsElementos.Tables[0].Rows)
				{
					elemento=new Entity.Pais();
					elemento.PaisId=int.Parse(ren["pais"].ToString());
					elemento.Descripcion=ren["descripcion"].ToString();
					elemento.UltimaActualizacion=int.Parse(ren["ultimaAct"].ToString());
					listaPaises.Add(elemento);
				}
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			listaPaises.AcceptChanges();
			return listaPaises;
		}

		/// <summary>
		/// Obtiene una Entity.Pais segn el parmetro recibido y lo carga a una entidad
		/// </summary>
		/// <param name="pais">System.String representa el pas</param>
		/// <returns>Entity.Pais</returns>
		public Entity.Pais TraerPaises(int pais)
		{	
			Entity.Pais elemento=null;
			DataSet dsElementos=null;
			try
			{	
				dsElementos=proc.Paises_Select(pais);
				foreach(System.Data.DataRow row in dsElementos.Tables[0].Rows)
				{
					elemento = new Entity.Pais();
					elemento.PaisId=int.Parse(row["pais"].ToString());
					elemento.Descripcion=row["descripcion"].ToString();
					elemento.UltimaActualizacion=int.Parse(row["UltimaAct"].ToString());
					elemento.AcceptChanges();
				}
			}
			catch(Exception ex)
			{
				Entity.LogController.Instance().AgregarMensaje(ex);
				throw;
			}
			return elemento;
		}
	}
}